#ifndef __GEN_H_
#define __GEN_H_

#include "testlib.h"
#include <vector>
#include <map>
#include <iostream>
#include <numeric>
#include <set>

template<typename T>
T random_element (const std::vector<T> &vec) {
  return vec[rnd.next(vec.size())];
}

struct Test {
  std::vector<std::pair<int, int>> edges;
};

std::vector<std::vector<int>> gen_partition (int n, int k) {
  std::vector<int> perm (n - 1, 0);
  std::iota(perm.begin(), perm.end(), 0);
  shuffle(perm.begin(), perm.end());

  std::set<int> breaks (perm.begin(), perm.begin() + k - 1);
  std::vector<std::vector<int>> ret (1, std::vector<int> (0));
  for (int i = 0; i < n; i++) {
    ret.back().push_back(i);
    if (breaks.count(i))
      ret.push_back(std::vector<int> (0));
  }

  return ret;
}

class Inflater {
public:
  virtual int inflate (int x) = 0;
};

class SparseInflater : public Inflater {
  int l, r;
  std::map<int, int> infl;
  std::set<int> used;

public:
  SparseInflater (int _l, int _r) : l(_l), r(_r) {
  }

  int inflate (int x) override {
    if (!infl.count(x)) {
      int y;
      do {
        y = rnd.next(l, r);
      } while (used.count(y));

      infl[x] = y;
      used.insert(y);
    }
    
    return infl[x];
  }
};

class DenseInflater : public Inflater {
  std::map<int, int> infl;
  std::vector<int> vals;
  
public:
  DenseInflater (int l, int r) {
    vals = std::vector<int> (0);
    for (int i = l; i <= r; i++) {
      vals.push_back(i);
    }

    shuffle(vals.begin(), vals.end());
  }

  int inflate (int x) override {
    if (!infl.count(x)) {
      infl[x] = vals.back();
      vals.pop_back();
    }

    return infl[x];
  }
};

void inflate_and_randomize (Test &test, int l, int r) {
  Inflater *infl = (r - l + 1 < 4 * (int) test.edges.size())
    ? (Inflater*) new DenseInflater(l, r)
    : (Inflater*) new SparseInflater(l, r);
  
  for (int i = 0; i < (int) test.edges.size(); i++) {
    test.edges[i].first = infl->inflate(test.edges[i].first);
    test.edges[i].second = infl->inflate(test.edges[i].second);

    if (rnd.next(2)) {
      std::swap(test.edges[i].first, test.edges[i].second);
    }
  }

  shuffle(test.edges.begin(), test.edges.end());
}

std::ostream& operator<< (std::ostream &out, const Test &test) {
  out << (int) test.edges.size() << '\n';
  for (int i = 0; i < (int) test.edges.size(); i++) {
    out << test.edges[i].first << " " << test.edges[i].second << '\n';
  }

  return out;
}

#endif
